﻿#light
module Anderscui.FsSamples.Common.ProcessHelper

open System
open System.Collections
open System.Diagnostics
open System.Management
open System.Text

/// Returns a string containing information on running processes.
let listAllProcesses() = 
    let sb = new StringBuilder()
    let mgmtClass = new ManagementClass("Win32_Process")
    
    for mo in mgmtClass.GetInstances() do
        sb.AppendLine("Name: \t" + mo.["Name"].ToString()) |> ignore
        sb.AppendLine("Id: \t" + mo.["ProcessId"].ToString()) |> ignore
        sb.AppendLine() |> ignore
        
    sb.ToString()
    
/// Returns a string containing information on running processes.
let listAllApplications() =
    let sb = new StringBuilder()
    
    Process.GetProcesses(".")
    |> Array.filter (fun p -> p.MainWindowTitle.Length > 0)
    |> Array.iter ( fun p ->
                    sb.AppendLine("Window Title: \t" + p.MainWindowTitle) |> ignore
                    sb.AppendLine("Process Name: \t" + p.ProcessName) |> ignore
                    sb.AppendLine("Window Handle: \t" + p.MainWindowHandle.ToString()) |> ignore
                    sb.AppendLine("Memory Allocation: \t" + p.PrivateMemorySize64.ToString()) |> ignore
                    sb.AppendLine() |> ignore)

    sb.ToString()
    
/// List all processes by image name.
let listAllByImageName() =
    let sb = new StringBuilder()
    let processes = Process.GetProcesses(".")
    try
        for p in processes do
            for pm in p.Modules do
                sb.AppendLine("Image Name: \t" + pm.ModuleName) |> ignore
                sb.AppendLine("File Path: \t" + pm.FileName) |> ignore
                sb.AppendLine("Memory Size: \t" + pm.ModuleMemorySize.ToString()) |> ignore
                sb.AppendLine("Version: \t" + pm.FileVersionInfo.FileVersion) |> ignore
                sb.AppendLine() |> ignore
    with
        | e -> printfn "%s" e.Message // TODO: event handling.
        
    sb.ToString()
          
/// Determine if a process is running by name.  
let checkForProcessByName (processName: string ) =
    let mgmtClass = new ManagementClass("Win32_Process")
    let moc = mgmtClass.GetInstances()
    let enumerator: IEnumerator = (moc :> IEnumerable).GetEnumerator()
    
    let mutable found = false
    while enumerator.MoveNext() && (not found) do
        let mo = enumerator.Current :?> ManagementObject
        if mo.["Name"].ToString().ToLower() = processName.ToLower() then
            found <- true
    
    found
    
let checkForProcessByImageName (processImageName: string) =
    Process.GetProcesses(".")
    |> Array.exists (fun p ->
        try
            let pms = p.Modules
            let enumerator = (pms :> IEnumerable).GetEnumerator()
            let mutable found = false
            while enumerator.MoveNext() && (not found) do
                let pm = enumerator.Current :?> ProcessModule
                if pm.ModuleName.ToLower() = processImageName.ToLower() then
                    found <- true
            found
        with
            | e -> false // TODO: event handling.
        )
        
/// Determine if an application is running by name
let checkForApplicationByName (appName: string) =
    Process.GetProcesses(".")
    |> Array.exists (fun p ->
        try
            p.ProcessName.ToLower() = appName.ToLower()    
        with
            | e -> false // TODO: event handling.
        )
        
/// Check for the existence of a process by ID; if the ID
/// is found, the method will return a true.
let findProcessById (processId: string)  = 
    let mgmtClass = new ManagementClass("Win32_Process")
    let moc = mgmtClass.GetInstances()
    let enumerator: IEnumerator = (moc :> IEnumerable).GetEnumerator()
    
    let mutable found = false
    while enumerator.MoveNext() && (not found) do
        let mo = enumerator.Current :?> ManagementObject
        if mo.["ProcessId"].ToString().ToLower() = processId.ToLower() then
            found <- true
            
    found