﻿#light
open System
open System.Drawing
open System.Windows.Forms

// 定义两个方法，此时仅仅声明函数的类型
type Shape =
    { reposition : Point -> unit;
      draw : Graphics -> unit }

// 创建一个Shape实例，并为它实现了两个方法
let makeShape initPos draw =
    let currPos = ref initPos in
    {   reposition = (fun newPos -> currPos := newPos);
        draw = (fun g -> draw !currPos g);
    }
    
// 创建一个circle（Shape）实例，它替换了前面draw的实现
let makeCircle initPos diam =
    makeShape initPos (fun pos g ->
        g.DrawEllipse(Pens.Blue, pos.X, pos.Y, diam, diam))
    
// 创建一个square（Shape）实例，它替换了前面draw的实现   
let makeSquare initPos size =
    makeShape initPos (fun pos g ->
        g.DrawRectangle(Pens.Blue, pos.X, pos.Y, size, size))

let getPoint(x, y) = new Point(x, y)

let shapes =
    [ makeCircle (getPoint(10, 10)) 20; makeSquare (getPoint(30, 30)) 20; ]
    
let mainForm =
    let form = new Form()
    let rand = new Random()
    form.Paint.Add(fun e ->
        shapes |> List.iter(fun s -> s.draw e.Graphics)
    )
    form.Click.Add(fun e ->
        shapes |> List.iter(fun s -> 
            s.reposition(new Point(rand.Next(form.Width), rand.Next(form.Height)));
            form.Invalidate())
    )
    form

[<STAThread>]
do Application.Run(mainForm)